#!/usr/bin/env python2.5
'''
Low Battery Nofication.py
 
Version History:
 	
 	v1.0 - Initial release
 
Description

	This script will loop through all indigo devices looking for devices that report
	battery levels. We do this by looking at the batteryLevel property that all
	devices have. If a device doesn't support battery reporting, that value will
	be 'None'. We also check to see if the device is enabled so that an easy way
	to shortcut getting notifications you don't want is to just disable the device.
	
	When the script finds one, it will use the following heuristics to decide
	if a notifiction should be sent for the device:
	
		- if the battery level is below the 'batteryThreshold' OR
		- if the device hasn't been updated in more than 'timeThreshold' days

	There are a couple of ways to set the thresholds. The default values are:
    
    batteryThreshold = 5%
    timeThreshold = 0 days (or anything over 24 hours)
	
	These defaults can be overridden globally by creating Indigo variables named:
		
		lbnBatteryThreshold
		lbnTimeThreshold
	
	If either of these are set, they will be used as the global thresholds. You may
	also use the Global Property Manager plugin (https://www.indigodomo.com/library/355/)
	to add some properties to specific devices that will override these thresholds:
		
		lbnBatteryThreshold
		lbnTimeThreshold
		lbnSkipNotification
		
	The latter tells the script to ignore this device (true/false). You might want to  
	use that for a battery powered device that can also be plugged into mains power. In 
	that case, it may or may not show battery level and may not actually report status 
	for long periods of time. This is a convenient way to just skip those devices.
	
	Once we've determined that a device needs a notification, then we simply look for an
	action group named with the device name + " low battery notification". Here's some 
	example names given the device name:
	
		Device Name        | Action Group Name
		-------------------+------------------------------
		Front Door Lock    | Front Door Lock low battery notification
		Hall motion sensor | Hall motion sensor low battery notification
		
	Note that the names are case sensitive so make sure they match. If the script finds
	a matching Action Group, it will execute it.
	
	There are several ways you can run this script: once a day is probably the easiest.
	You can get clever however and run it every hour. Add some current time conditions to
	the schedule and it can notify you once an hour between, say, 6pm and 10pm. Lots of
	options.
'''     

try:
	import indigo
except ImportError:
	print "Indigo control scripts can only be used from within Indigo"
	raise ImportError
from datetime import datetime, timedelta

## Convenience log function
def log(msg):
	indigo.server.log(msg, type="Low Battery Notification Script")

## Constants and global variables
kTrueStringValues = ['true', '1', 't', 'y', 'yes']
notifications = 0

## Get default overrides from variables
# Battery Threshold
v = indigo.variables.get("lbnBatteryThreshold", None)
try:
	batteryThreshold = int(v.value)
except:
	# default to 5%
	batteryThreshold = 5

# Time Threshold
v = indigo.variables.get("lbnTimeThreshold", None)
try:
	timeThreshold = int(v.value)
except:
	# default to anything older than 24 hours
	timeThreshold = 0
	
## Start main part of script
log("Starting battery notification script with default battery threshold of %i%% and time threshold of %i days" % (batteryThreshold, timeThreshold))
## Loop through devices looking for devices that have battery levels
for device in indigo.devices:
	if device.batteryLevel != None and device.enabled:
		skipNotification = False
		days_since_last_update = (datetime.now() - device.lastChanged).days
		# get any device specific props added using the Global Property Manager plugin
		deviceSpecificProps = device.globalProps.get("com.indigodomo.indigoserver", None)
		if deviceSpecificProps:
			# if there were some props there, get the ones we care about or just use the defaults
			try: batteryThreshold = int(deviceSpecificProps.get("lbnBatteryThreshold", batteryThreshold))
			except: pass
			try: timeThreshold = int(deviceSpecificProps.get("lbnTimeThreshold", timeThreshold))
			except: pass
			skipNotificationString = deviceSpecificProps.get("lbnSkipNotification", "false")
			# We're trying to be forgiving in terms of what's entered for the skip field, so we lowercase
			# it and test it against the list defined above.
			if skipNotificationString.lower() in kTrueStringValues:
				skipNotification = True
		# Here's the meat: if the battery crosses the threshold or the last update does and we didn't find a skip flag
		if (device.batteryLevel < batteryThreshold or days_since_last_update > timeThreshold) and not skipNotification:
			log("Threshold exceeded for '%s' using battery threshold of %i%% and time threshold of %i days..." % (device.name, batteryThreshold, timeThreshold))
			notifications += 1
			# Try to get an appropriately named action group. If we find one then execute it.
			ag = indigo.actionGroups.get(device.name+" low battery notification", None)
			if ag:
				log("    ...executing notification action group")
				indigo.actionGroup.execute(ag)
			else:
				log("    ...no notification action group found")

# If we didn't find any low batteries, wrap up with a nice log message.
if notifications == 0:
	log("No low batteries found")
